@@ -35,6 +35,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA
 #include <klocale.h>
 #include <kpixmapsequence.h>
 #include <kpixmapsequencewidget.h>
+#include <kstandarddirs.h>
 #include <kurl.h>
 
 // Local
@@ -44,6 +45,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA
 #include <lib/documentview/imageviewadapter.h>
 #include <lib/documentview/svgviewadapter.h>
 #include <lib/documentview/videoviewadapter.h>
+#include <lib/gwenviewconfig.h>
 #include <lib/mimetypeutils.h>
 #include <lib/signalblocker.h>
 #include <lib/slideshow.h>
@@ -71,6 +73,8 @@ struct DocumentViewPrivate {
 	KActionCollection* mActionCollection;
 	ZoomWidget* mZoomWidget;
 	KAction* mZoomToFitAction;
+	QCursor mZoomCursor;
+	QCursor mPreviousCursor;
 
 	KPixmapSequenceWidget* mLoadingIndicator;
 
@@ -119,6 +123,26 @@ struct DocumentViewPrivate {
 			that, SLOT(slotZoomWidgetChanged(qreal)) );
 	}
 
+	void setupZoomCursor() {
+		QString path = KStandardDirs::locate("appdata", "cursors/zoom.png");
+		QPixmap cursorPixmap = QPixmap(path);
+		mZoomCursor = QCursor(cursorPixmap);
+	}
+
+	void setZoomCursor() {
+		mAdapter->widget()->grabKeyboard();
+		QCursor currentCursor = mAdapter->cursor();
+		if (currentCursor.pixmap().cacheKey() != mZoomCursor.pixmap().cacheKey()) {
+			mPreviousCursor = currentCursor;
+		}
+		mAdapter->setCursor(mZoomCursor);
+	}
+
+	void restoreCursor() {
+		mAdapter->widget()->releaseKeyboard();
+		mAdapter->setCursor(mPreviousCursor);
+	}
+
 	void setupZoomActions() {
 		KActionCategory* view=new KActionCategory(i18nc("@title actions category - means actions changing smth in interface","View"), mActionCollection);
 
@@ -265,6 +289,95 @@ struct DocumentViewPrivate {
 		}
 		mLoadingIndicator->hide();
 	}
+
+
+	bool adapterMousePressEventFilter(QMouseEvent* event) {
+		if (mAdapter->canZoom()) {
+			if (event->modifiers() == Qt::ControlModifier) {
+				// Ctrl + Left or right button => zoom in or out
+				setZoomCursor();
+				if (event->button() == Qt::LeftButton) {
+					that->zoomIn(event->pos());
+				} else if (event->button() == Qt::RightButton) {
+					that->zoomOut(event->pos());
+				}
+				return true;
+			} else if (event->button() == Qt::MidButton) {
+				// Middle click => toggle zoom to fit
+				mZoomToFitAction->trigger();
+				return true;
+			}
+		}
+		return false;
+	}
+
+	bool adapterMouseReleaseEventFilter(QMouseEvent* event) {
+		if (mAdapter->canZoom() && event->modifiers() == Qt::ControlModifier) {
+			// Eat the mouse release so that the svg view does not restore its
+			// drag cursor on release: we want the zoom cursor to stay as long
+			// as Control is held down.
+			return true;
+		}
+		return false;
+	}
+
+	bool adapterMouseDoubleClickEventFilter(QMouseEvent* event) {
+		if (event->modifiers() == Qt::NoModifier) {
+			that->toggleFullScreenRequested();
+			return true;
+		}
+		return false;
+	}
+
+
+	bool adapterWheelEventFilter(QWheelEvent* event) {
+		if (mAdapter->canZoom() && event->modifiers() & Qt::ControlModifier) {
+			// Ctrl + wheel => zoom in or out
+			setZoomCursor();
+			if (event->delta() > 0) {
+				that->zoomIn(event->pos());
+			} else {
+				that->zoomOut(event->pos());
+			}
+			return true;
+		}
+		if (event->modifiers() == Qt::NoModifier
+			&& GwenviewConfig::mouseWheelBehavior() == MouseWheelBehavior::Browse
+			) {
+			// Browse with mouse wheel
+			if (event->delta() > 0) {
+				that->previousImageRequested();
+			} else {
+				that->nextImageRequested();
+			}
+			return true;
+		}
+		return false;
+	}
+
+
+	bool adapterContextMenuEventFilter(QContextMenuEvent* event) {
+		// Filter out context menu if Ctrl is down to avoid showing it when
+		// zooming out with Ctrl + Right button
+		if (event->modifiers() == Qt::ControlModifier) {
+			return true;
+		}
+		return false;
+	}
+
+	bool adapterKeyPressEventFilter(QKeyEvent* event) {
+		if (mAdapter->canZoom() && event->modifiers() == Qt::ControlModifier) {
+			setZoomCursor();
+		}
+		return false;
+	}
+
+	bool adapterKeyReleaseEventFilter(QKeyEvent* event) {
+		if (mAdapter->canZoom() && event->modifiers() != Qt::ControlModifier) {
+			restoreCursor();
+		}
+		return false;
+	}
 };
 
 
@@ -281,6 +394,7 @@ DocumentView::DocumentView(QWidget* parent, SlideShow* slideShow, KActionCollect
 	d->mZoomWidgetVisible = true;
 	d->setupZoomWidget();
 	d->setupZoomActions();
+	d->setupZoomCursor();
 	d->setCurrentAdapter(new MessageViewAdapter(this));
 }
 
@@ -387,6 +501,10 @@ void DocumentView::finishOpenUrl() {
 		SLOT(slotLoadingFailed()) );
 	d->mAdapter->setDocument(d->mDocument);
 	d->updateCaption();
+
+	if (d->mDocument->loadingState() == Document::Loaded) {
+		slotLoaded();
+	}
 }
 
 
@@ -479,20 +597,21 @@ void DocumentView::slotZoomWidgetChanged(qreal zoom) {
 
 bool DocumentView::eventFilter(QObject*, QEvent* event) {
 	if (event->type() == QEvent::MouseButtonPress) {
-		// Middle click => toggle zoom to fit
-		QMouseEvent* mouseEvent = static_cast<QMouseEvent*>(event);
-		if (mouseEvent->button() == Qt::MidButton) {
-			d->mZoomToFitAction->trigger();
-			return true;
-		}
+		return d->adapterMousePressEventFilter(static_cast<QMouseEvent*>(event));
+	} else if (event->type() == QEvent::MouseButtonRelease) {
+		return d->adapterMouseReleaseEventFilter(static_cast<QMouseEvent*>(event));
 	} else if (event->type() == QEvent::Resize) {
 		d->updateZoomSnapValues();
 	} else if (event->type() == QEvent::MouseButtonDblClick) {
-		QMouseEvent* mouseEvent = static_cast<QMouseEvent*>(event);
-		if (mouseEvent->modifiers() == Qt::NoModifier) {
-			emit toggleFullScreenRequested();
-			return true;
-		}
+		return d->adapterMouseDoubleClickEventFilter(static_cast<QMouseEvent*>(event));
+	} else if (event->type() == QEvent::Wheel) {
+		return d->adapterWheelEventFilter(static_cast<QWheelEvent*>(event));
+	} else if (event->type() == QEvent::ContextMenu) {
+		return d->adapterContextMenuEventFilter(static_cast<QContextMenuEvent*>(event));
+	} else if (event->type() == QEvent::KeyPress) {
+		return d->adapterKeyPressEventFilter(static_cast<QKeyEvent*>(event));
+	} else if (event->type() == QEvent::KeyRelease) {
+		return d->adapterKeyReleaseEventFilter(static_cast<QKeyEvent*>(event));
 	}
 
 	return false;
