@@ -5,6 +5,7 @@
  *  Copyright (C) 2003 Nadeem Hasan <nhasan@kde.org>
  *  Copyright (C) 2004 Bernd Brandstetter <bbrand@freenet.de>
  *  Copyright (C) 2006 Urs Wolfer <uwolfer @ kde.org>
+ *  Copyright (C) 2010 Martin Gräßlin <kde@martin-graesslin.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -28,6 +29,15 @@
 #include <QShortcut>
 #include <QMenu>
 #include <QDesktopWidget>
+#include <QVarLengthArray>
+#include <QCloseEvent>
+#include <QDrag>
+#include <QMouseEvent>
+#include <QPainter>
+#include <QtCore/QXmlStreamReader>
+#include <QtDBus/QDBusConnection>
+#include <QtDBus/QDBusConnectionInterface>
+#include <QtDBus/QDBusInterface>
 
 #include <klocale.h>
 
@@ -54,9 +64,21 @@
 #include <qdebug.h>
 
 #include "regiongrabber.h"
+#include "freeregiongrabber.h"
 #include "windowgrabber.h"
+#include "ksnapshotpreview.h"
 #include "ui_ksnapshotwidget.h"
 
+#ifdef KIPI_FOUND
+#include <libkipi/plugin.h>
+#include "kipiinterface.h"
+#include <KAction>
+#endif
+
+#ifdef HAVE_X11_EXTENSIONS_XFIXES_H
+#include <X11/extensions/Xfixes.h>
+#include <QX11Info>
+#endif
 
 class KSnapshotWidget : public QWidget, public Ui::KSnapshotWidget
 {
@@ -66,21 +88,23 @@ class KSnapshotWidget : public QWidget, public Ui::KSnapshotWidget
         {
             setupUi(this);
             btnNew->setIcon(KIcon("ksnapshot"));
-            btnSave->setIcon(KIcon("document-save"));
-            btnOpen->setIcon(KIcon("document-open"));
-            btnCopy->setIcon(KIcon("edit-copy"));
         }
 };
 
 KSnapshot::KSnapshot(QWidget *parent,  KSnapshotObject::CaptureMode mode )
   : KDialog(parent), KSnapshotObject(), modified(false), savedPosition(QPoint(-1, -1))
 {
+    // TEMPORARY Make sure "untitled" enters the string freeze for 4.6, 
+    // as explained in http://lists.kde.org/?l=kde-graphics-devel&m=128942871430175&w=2
+    const QString untitled = QString(i18n("untitled"));
+    
     setCaption( "" );
-    setModal( true );
     showButtonSeparator( true );
-    setButtons(Help|User1);
-    setButtonGuiItem( User1, KStandardGuiItem::quit() );
-    setDefaultButton( User1 );
+    setButtons(Help | Apply | User1 | User2);
+    setButtonGuiItem(Apply, KStandardGuiItem::saveAs());
+    setButtonGuiItem(User1, KGuiItem(i18n("Copy"), "edit-copy"));
+    setButtonGuiItem(User2, KGuiItem(i18n("Send To..."), "document-open"));
+    setDefaultButton(Apply);
     grabber = new QWidget( 0,  Qt::X11BypassWindowManagerHint );
     grabber->move( -1000, -1000 );
     grabber->installEventFilter( this );
@@ -93,72 +117,122 @@ KSnapshot::KSnapshot(QWidget *parent,  KSnapshotObject::CaptureMode mode )
 
     mainWidget = new KSnapshotWidget( vbox );
 
-    connect( mainWidget->lblImage, SIGNAL( startDrag() ), SLOT( slotDragSnapshot() ) );
-    connect( mainWidget->btnNew, SIGNAL( clicked() ), SLOT( slotGrab() ) );
-    connect( mainWidget->btnSave, SIGNAL( clicked() ), SLOT( slotSaveAs() ) );
-    connect( mainWidget->btnCopy, SIGNAL( clicked() ), SLOT( slotCopy() ) );
-//    connect( mainWidget->btnOpen, SIGNAL( clicked() ), SLOT( slotOpen() ) );
-    connect( mainWidget->comboMode, SIGNAL( activated(int) ), SLOT( slotModeChanged(int) ) );
+    connect(mainWidget->lblImage, SIGNAL(startDrag()), SLOT(slotDragSnapshot()));
+    connect(mainWidget->btnNew, SIGNAL(clicked()), SLOT(slotGrab()));
+    connect(this, SIGNAL(applyClicked()), SLOT(slotSaveAs()));
+    connect(this, SIGNAL(user1Clicked()), SLOT(slotCopy()));
+    connect(mainWidget->comboMode, SIGNAL(activated(int)), SLOT(slotModeChanged(int)));
+
+    if (qApp->desktop()->numScreens() < 2) {
+        mainWidget->comboMode->removeItem(CurrentScreen);
+    }
 
     openMenu = new QMenu(this);
-    mainWidget->btnOpen->setMenu(openMenu);
-    connect(openMenu, SIGNAL(aboutToShow()),
-            this, SLOT(slotPopulateOpenMenu()));
-    connect(openMenu, SIGNAL(triggered(QAction*)),
-            this, SLOT(slotOpen(QAction*)));
+    button(User2)->setMenu(openMenu);
+    connect(openMenu, SIGNAL(aboutToShow()), this, SLOT(slotPopulateOpenMenu()));
+    connect(openMenu, SIGNAL(triggered(QAction*)), this, SLOT(slotOpen(QAction*)));
 
     mainWidget->spinDelay->setSuffix(ki18np(" second", " seconds"));
 
     grabber->show();
-    grabber->grabMouse( Qt::WaitCursor );
+    grabber->grabMouse();
+
+    KConfigGroup conf(KGlobal::config(), "GENERAL");
+
+#ifdef KIPI_FOUND
+    mPluginLoader = new KIPI::PluginLoader(QStringList(), new KIPIInterface(this), "");
+#endif
+
+#ifdef HAVE_X11_EXTENSIONS_XFIXES_H
+    {
+        int tmp1, tmp2;
+        //Check whether the XFixes extension is available
+        Display *dpy = QX11Info::display();
+        if (!XFixesQueryExtension( dpy, &tmp1, &tmp2 )) {
+            mainWidget->cbIncludePointer->hide();
+            mainWidget->lblIncludePointer->hide();
+        }
+    }
+#else
+    mainWidget->cbIncludePointer->hide();
+    mainWidget->lblIncludePointer->hide();
+#endif
+    setIncludePointer(conf.readEntry("includePointer", false));
+
+    // check if kwin screenshot effect is available
+    includeAlpha = false;
+    if ( QDBusConnection::sessionBus().interface()->isServiceRegistered( "org.kde.kwin" ) ) {
+        QDBusInterface kwinInterface( "org.kde.kwin", "/", "org.freedesktop.DBus.Introspectable" );
+        QDBusReply<QString> reply = kwinInterface.call( "Introspect" );
+        if ( reply.isValid() ) {
+            QXmlStreamReader xml( reply.value() );
+            while ( !xml.atEnd() ) {
+                xml.readNext();
+                if ( xml.tokenType() == QXmlStreamReader::StartElement &&
+                    xml.name().toString() == "node" ) {
+                    if ( xml.attributes().value( "name" ).toString() == "Screenshot" ) {
+                        includeAlpha = true;
+                        break;
+                    }
+                }
+            }
+        }
+    }
 
     kDebug() << "Mode = " << mode;
-    if ( mode == KSnapshotObject::FullScreen )
+    if ( mode == KSnapshotObject::FullScreen ) {
         snapshot = QPixmap::grabWindow( QApplication::desktop()->winId() );
+#ifdef HAVE_X11_EXTENSIONS_XFIXES_H
+        if ( haveXFixes && includePointer() )
+            grabPointerImage(0, 0);
+#endif
+    }
     else if ( mode == KSnapshotObject::CurrentScreen ) {
-	kDebug() << "Desktop Geom = " << QApplication::desktop()->geometry();
-	QDesktopWidget *desktop = QApplication::desktop();
-	int screenId = desktop->screenNumber( QCursor::pos() );
-	kDebug() << "Screenid = " << screenId;
-	QRect geom = desktop->screenGeometry( screenId );
-	kDebug() << "Geometry = " << screenId;
-	snapshot = QPixmap::grabWindow( desktop->winId(),
-					geom.x(), geom.y(), geom.width(), geom.height() );
-
+        kDebug() << "Desktop Geom = " << QApplication::desktop()->geometry();
+        QDesktopWidget *desktop = QApplication::desktop();
+        int screenId = desktop->screenNumber( QCursor::pos() );
+        kDebug() << "Screenid = " << screenId;
+        QRect geom = desktop->screenGeometry( screenId );
+        kDebug() << "Geometry = " << screenId;
+        snapshot = QPixmap::grabWindow( desktop->winId(),
+                geom.x(), geom.y(), geom.width(), geom.height() );
     }
     else {
         setMode( mode );
-	switch(mode)
-	{
+        switch(mode)
+        {
             case KSnapshotObject::WindowUnderCursor:
-	   {
-		setIncludeDecorations( true );
-		performGrab();
-		break;
-	   }
-	   case  KSnapshotObject::ChildWindow:
-	   {
-	  	slotGrab();
-		break;
-	   }
-	   case KSnapshotObject::Region:
-	   {
-	        grabRegion();
-	        break;
-	   }
-	   default:
-	        break;
-	  }
+                {
+                    setIncludeDecorations( true );
+                    performGrab();
+                    break;
+                }
+            case  KSnapshotObject::ChildWindow:
+                {
+                    slotGrab();
+                    break;
+                }
+            case KSnapshotObject::Region:
+                {
+                    grabRegion();
+                    break;
+                }
+            case KSnapshotObject::FreeRegion:
+            {
+                 grabFreeRegion();
+                 break;
+            }
+            default:
+                break;
+        }
     }
 
     //When we use argument to take snapshot we mustn't hide it.
-    if(mode !=  KSnapshotObject::ChildWindow)
-    {
+    if (mode !=  KSnapshotObject::ChildWindow) {
        grabber->releaseMouse();
        grabber->hide();
     }
 
-    KConfigGroup conf(KGlobal::config(), "GENERAL");
     setDelay( conf.readEntry("delay", 0) );
     setMode( conf.readEntry("mode", 0) );
     setIncludeDecorations(conf.readEntry("includeDecorations",true));
@@ -183,19 +257,20 @@ KSnapshot::KSnapshot(QWidget *parent,  KSnapshotObject::CaptureMode mode )
 
     new QShortcut( Qt::Key_Q, this, SLOT(slotSave()));
 
-    new QShortcut( KStandardShortcut::shortcut( KStandardShortcut::Copy ).primary(), mainWidget->btnCopy, SLOT(animateClick()));
+    new QShortcut( KStandardShortcut::shortcut( KStandardShortcut::Copy ).primary(), button(User1), SLOT(animateClick()));
 
-    new QShortcut( KStandardShortcut::shortcut( KStandardShortcut::Save ).primary(), mainWidget->btnSave, SLOT(animateClick()));
-    new QShortcut( Qt::Key_S, mainWidget->btnSave, SLOT(animateClick()));
+    new QShortcut( KStandardShortcut::shortcut( KStandardShortcut::Save ).primary(), button(Apply), SLOT(animateClick()));
+    new QShortcut( Qt::Key_S, button(Apply), SLOT(animateClick()));
 
     new QShortcut( KStandardShortcut::shortcut( KStandardShortcut::New ).primary(), mainWidget->btnNew, SLOT(animateClick()) );
     new QShortcut( Qt::Key_N, mainWidget->btnNew, SLOT(animateClick()) );
     new QShortcut( Qt::Key_Space, mainWidget->btnNew, SLOT(animateClick()) );
 
-    setEscapeButton( User1 );
-    connect( this, SIGNAL( user1Clicked() ), SLOT( reject() ) );
-
     mainWidget->btnNew->setFocus();
+    setInitialSize(QSize(250, 500));
+
+    KConfigGroup cg(KGlobal::config(), "MainWindow");
+    restoreDialogSize(cg);
 }
 
 KSnapshot::~KSnapshot()
@@ -212,7 +287,7 @@ void KSnapshot::resizeEvent( QResizeEvent * )
 void KSnapshot::slotSave()
 {
     // Make sure the name is not already being used
-    while (KIO::NetAccess::exists( filename, KIO::NetAccess::DestinationSide, this )) {
+    while(KIO::NetAccess::exists( filename, KIO::NetAccess::DestinationSide, this )) {
         autoincFilename();
     }
 
@@ -225,24 +300,29 @@ void KSnapshot::slotSave()
 
 void KSnapshot::slotSaveAs()
 {
+    QStringList mimetypes = KImageIO::mimeTypes( KImageIO::Writing );
+
     // Make sure the name is not already being used
-    while (KIO::NetAccess::exists( filename, KIO::NetAccess::DestinationSide, this )) {
+    while(KIO::NetAccess::exists( filename, KIO::NetAccess::DestinationSide, this )) {
         autoincFilename();
     }
 
-    QStringList mimetypes = KImageIO::mimeTypes( KImageIO::Writing );
-    KFileDialog dlg( filename.url(), mimetypes.join(" "), this);
+    QPointer<KFileDialog> dlg = new KFileDialog( filename.url(), mimetypes.join(" "), this);
 
-    dlg.setOperationMode( KFileDialog::Saving );
-    dlg.setCaption( i18n("Save As") );
-    dlg.setSelection( filename.url() );
+    dlg->setOperationMode( KFileDialog::Saving );
+    dlg->setCaption( i18n("Save As") );
+    dlg->setSelection( filename.url() );
 
-    if ( !dlg.exec() )
+    if ( !dlg->exec() ) {
+        delete dlg;
         return;
+    }
 
-    KUrl url = dlg.selectedUrl();
-    if ( !url.isValid() )
+    KUrl url = dlg->selectedUrl();
+    if ( !url.isValid() ){
+	delete dlg;
         return;
+    }
 
     if ( save(url,this) ) {
         filename = url;
@@ -250,6 +330,8 @@ void KSnapshot::slotSaveAs()
         autoincFilename();
         updateCaption();
     }
+
+    delete dlg;
 }
 
 void KSnapshot::slotCopy()
@@ -264,6 +346,7 @@ void KSnapshot::slotDragSnapshot()
 
     drag->setMimeData(new QMimeData);
     drag->mimeData()->setImageData(snapshot);
+    drag->mimeData()->setData("application/x-kde-suggestedfilename", filename.fileName().toUtf8());
     drag->setPixmap(preview());
     drag->start();
 }
@@ -273,18 +356,26 @@ void KSnapshot::slotGrab()
     savedPosition = pos();
     hide();
 
-    if ( delay() ) {
+    if (delay()) {
         //kDebug() << "starting timer with time of" << delay();
-        grabTimer.start( delay());
+        grabTimer.start(delay());
     }
     else {
-        if ( mode() == Region ) {
-            grabRegion();
-        }
-        else {
-            grabber->show();
-            grabber->grabMouse( Qt::CrossCursor );
-        }
+        QTimer::singleShot(0, this, SLOT(startUndelayedGrab()));
+    }
+}
+
+void KSnapshot::startUndelayedGrab()
+{
+    if (mode() == Region) {
+        grabRegion();
+    }
+    else if ( mode() == FreeRegion ) {
+        grabFreeRegion();
+    }
+    else {
+        grabber->show();
+        grabber->grabMouse(Qt::CrossCursor);
     }
 }
 
@@ -349,19 +440,22 @@ void KSnapshot::slotOpen(QAction* action)
     KService::Ptr service = serviceAction->service;
     if (!service)
     {
-        KOpenWithDialog dlg(list, this);
-        if (!dlg.exec())
+        QPointer<KOpenWithDialog> dlg = new KOpenWithDialog(list, this);
+        if (!dlg->exec())
         {
+            delete dlg;
             return;
         }
 
-        service = dlg.service();
+        service = dlg->service();
 
-        if (!service && !dlg.text().isEmpty())
+        if (!service && !dlg->text().isEmpty())
         {
-             KRun::run(dlg.text(), list, this);
+             KRun::run(dlg->text(), list, this);
+             delete dlg;
              return;
         }
+	delete dlg;
     }
 
     // we have an action with a service, run it!
@@ -377,7 +471,7 @@ void KSnapshot::slotPopulateOpenMenu()
         currentAction->deleteLater();
     }
 
-    const KService::List services = KMimeTypeTrader::self()->query( "image/png");
+    const KService::List services = KMimeTypeTrader::self()->query("image/png");
     QMap<QString, KService::Ptr> apps;
 
     foreach (const KService::Ptr &service, services)
@@ -387,12 +481,51 @@ void KSnapshot::slotPopulateOpenMenu()
 
     foreach (const KService::Ptr &service, apps)
     {
-        QString name = service->name().replace( "&", "&&" );
+        QString name = service->name().replace( '&', "&&" );
         openMenu->addAction(new KSnapshotServiceAction(service,
                                                        KIcon(service->icon()),
                                                        name, this));
     }
 
+
+#ifdef KIPI_FOUND
+    KIPI::PluginLoader::PluginList pluginList = mPluginLoader->pluginList();
+
+    Q_FOREACH(KIPI::PluginLoader::Info* pluginInfo, pluginList) {
+        if (!pluginInfo->shouldLoad()) {
+            continue;
+        }
+        KIPI::Plugin* plugin = pluginInfo->plugin();
+        if (!plugin) {
+            kWarning() << "Plugin from library" << pluginInfo->library() << "failed to load";
+            continue;
+        }
+
+        plugin->setup(this);
+
+        QList<KAction*> actions = plugin->actions();
+        QSet<KAction*> exportActions;
+        Q_FOREACH(KAction* action, actions) {
+            KIPI::Category category = plugin->category(action);
+            if(category == KIPI::ExportPlugin) {
+                exportActions << action;
+            } else if (category == KIPI::ImagesPlugin) {
+                // Horrible hack. Why are the print images and the e-mail images plugins in the same category as rotate and edit metadata!?
+                if( pluginInfo->library().contains("kipiplugin_printimages") || pluginInfo->library().contains("kipiplugin_sendimages")) {
+                    exportActions << action;
+                }
+            }
+        }
+
+        Q_FOREACH(KAction* action, exportActions) {
+            openMenu->addAction(action);
+        }
+
+//        FIXME: Port
+//            plugin->actionCollection()->readShortcutSettings();
+    }
+#endif
+
     openMenu->addSeparator();
     KService::Ptr none;
     openMenu->addAction(new KSnapshotServiceAction(none,
@@ -410,7 +543,14 @@ void KSnapshot::slotRegionGrabbed( const QPixmap &pix )
     updateCaption();
   }
 
-  rgnGrab->deleteLater();
+  if( mode() == KSnapshotObject::Region )
+  {
+    rgnGrab->deleteLater();
+  }
+  else if( mode() == KSnapshotObject::FreeRegion ) {
+    freeRgnGrab->deleteLater();
+  }
+
   QApplication::restoreOverrideCursor();
   show();
 }
@@ -429,15 +569,27 @@ void KSnapshot::slotWindowGrabbed( const QPixmap &pix )
     show();
 }
 
+void KSnapshot::slotScreenshotReceived( qulonglong handle )
+{
+#ifdef Q_WS_X11
+    slotWindowGrabbed( QPixmap::fromX11Pixmap( handle ) );
+#endif
+}
+
 void KSnapshot::closeEvent( QCloseEvent * e )
 {
     KConfigGroup conf(KGlobal::config(), "GENERAL");
-    conf.writeEntry("delay",delay());
-    conf.writeEntry("mode",mode());
-    conf.writeEntry("includeDecorations",includeDecorations());
+    conf.writeEntry("delay", delay());
+    conf.writeEntry("mode", mode());
+    conf.writeEntry("includeDecorations", includeDecorations());
+    conf.writeEntry("includePointer", includePointer());
+
+    KConfigGroup cg(KGlobal::config(), "MainWindow");
+    saveDialogSize(cg);
+
     KUrl url = filename;
-    url.setPass( QString::null );	//krazy:exclude=nullstrassign for old broken gcc
-    conf.writePathEntry("filename",url.url());
+    url.setPass(QString::null); //krazy:exclude=nullstrassign for old broken gcc
+    conf.writePathEntry("filename", url.url());
 
     conf.sync();
     e->accept();
@@ -468,10 +620,21 @@ void KSnapshot::grabRegion()
 
 }
 
+void KSnapshot::grabFreeRegion()
+{
+   freeRgnGrab = new FreeRegionGrabber();
+   connect( freeRgnGrab, SIGNAL( freeRegionGrabbed( const QPixmap & ) ),
+                     SLOT( slotRegionGrabbed( const QPixmap & ) ) );
+
+}
+
 void KSnapshot::grabTimerDone()
 {
     if ( mode() == Region ) {
-       grabRegion();
+        grabRegion();
+    }
+    else if ( mode() == FreeRegion ) {
+        grabFreeRegion();
     }
     else {
         performGrab();
@@ -481,42 +644,79 @@ void KSnapshot::grabTimerDone()
 
 void KSnapshot::performGrab()
 {
+    int x = 0;
+    int y = 0;
+
     grabber->releaseMouse();
     grabber->hide();
     grabTimer.stop();
 
-    title = QString();
-    windowClass = QString();
+    title.clear();
+    windowClass.clear();
 
     if ( mode() == ChildWindow ) {
         WindowGrabber wndGrab;
         connect( &wndGrab, SIGNAL( windowGrabbed( const QPixmap & ) ),
                            SLOT( slotWindowGrabbed( const QPixmap & ) ) );
         wndGrab.exec();
+        QPoint offset = wndGrab.lastWindowPosition();
+        x = offset.x();
+        y = offset.y();
+        qDebug() << "last window position is" << offset;
     }
     else if ( mode() == WindowUnderCursor ) {
-        snapshot = WindowGrabber::grabCurrent( includeDecorations() );
+        if ( includeAlpha ) {
+            // use kwin effect
+            QDBusConnection::sessionBus().connect("org.kde.kwin", "/Screenshot",
+                                                  "org.kde.kwin.Screenshot", "screenshotCreated",
+                                                  this, SLOT(slotScreenshotReceived(qulonglong)));
+            QDBusInterface interface( "org.kde.kwin", "/Screenshot", "org.kde.kwin.Screenshot" );
+            int mask = 0;
+            if ( includeDecorations() )
+            {
+                mask |= 1 << 0;
+            }
+            if ( includePointer() )
+            {
+                mask |= 1 << 1;
+            }
+            interface.call( "screenshotWindowUnderCursor", mask );
+        } else {
+            snapshot = WindowGrabber::grabCurrent( includeDecorations() );
 
-	// If we're showing decorations anyway then we'll add the title and window
-	// class to the output image meta data.
-	if ( includeDecorations() ) {
-	    title = WindowGrabber::lastWindowTitle();
-	    windowClass = WindowGrabber::lastWindowClass();
-	}
+            QPoint offset = WindowGrabber::lastWindowPosition();
+            x = offset.x();
+            y = offset.y();
+
+            // If we're showing decorations anyway then we'll add the title and window
+            // class to the output image meta data.
+            if ( includeDecorations() ) {
+                title = WindowGrabber::lastWindowTitle();
+                windowClass = WindowGrabber::lastWindowClass();
+            }
+        }
     }
     else if ( mode() == CurrentScreen ) {
-	kDebug() << "Desktop Geom2 = " << QApplication::desktop()->geometry();
-	QDesktopWidget *desktop = QApplication::desktop();
-	int screenId = desktop->screenNumber( QCursor::pos() );
-	kDebug() << "Screenid2 = " << screenId;
-	QRect geom = desktop->screenGeometry( screenId );
-	kDebug() << "Geometry2 = " << geom;
-	snapshot = QPixmap::grabWindow( desktop->winId(),
-					geom.x(), geom.y(), geom.width(), geom.height() );
+        kDebug() << "Desktop Geom2 = " << QApplication::desktop()->geometry();
+        QDesktopWidget *desktop = QApplication::desktop();
+        int screenId = desktop->screenNumber( QCursor::pos() );
+        kDebug() << "Screenid2 = " << screenId;
+        QRect geom = desktop->screenGeometry( screenId );
+        kDebug() << "Geometry2 = " << geom;
+        x = geom.x();
+        y = geom.y();
+        snapshot = QPixmap::grabWindow( desktop->winId(),
+                x, y, geom.width(), geom.height() );
     }
     else {
         snapshot = QPixmap::grabWindow( QApplication::desktop()->winId() );
     }
+#ifdef HAVE_X11_EXTENSIONS_XFIXES_H
+    if (haveXFixes && includePointer()) {
+        grabPointerImage(x, y);
+    }
+#endif
+
     updatePreview();
     QApplication::restoreOverrideCursor();
     modified = true;
@@ -527,6 +727,35 @@ void KSnapshot::performGrab()
     show();
 }
 
+void KSnapshot::grabPointerImage(int offsetx, int offsety)
+// Uses the X11_EXTENSIONS_XFIXES_H extension to grab the pointer image, and overlays it onto the snapshot.
+{
+#ifdef HAVE_X11_EXTENSIONS_XFIXES_H
+    XFixesCursorImage *xcursorimg = XFixesGetCursorImage( QX11Info::display() );
+    if ( !xcursorimg )
+      return;
+
+    //Annoyingly, xfixes specifies the data to be 32bit, but places it in an unsigned long *
+    //which can be 64 bit.  So we need to iterate over a 64bit structure to put it in a 32bit
+    //structure.
+    QVarLengthArray< quint32 > pixels( xcursorimg->width * xcursorimg->height );
+    for (int i = 0; i < xcursorimg->width * xcursorimg->height; ++i)
+        pixels[i] = xcursorimg->pixels[i] & 0xffffffff;
+
+    QImage qcursorimg((uchar *) pixels.data(), xcursorimg->width, xcursorimg->height,
+                       QImage::Format_ARGB32_Premultiplied);
+
+    QPainter painter(&snapshot);
+    painter.drawImage(QPointF(xcursorimg->x - xcursorimg->xhot - offsetx, xcursorimg->y - xcursorimg ->yhot - offsety), qcursorimg);
+
+    XFree(xcursorimg);
+#else
+    Q_UNUSED(offsetx);
+    Q_UNUSED(offsety);
+    return;
+#endif
+}
+
 void KSnapshot::setTime(int newTime)
 {
     setDelay(newTime);
@@ -574,18 +803,19 @@ void KSnapshot::exit()
 
 void KSnapshot::slotModeChanged(int mode)
 {
+    mainWidget->cbIncludePointer->setEnabled(!(mode == Region || mode == FreeRegion));
+    mainWidget->lblIncludePointer->setEnabled(!(mode == Region || mode == FreeRegion));
     mainWidget->cbIncludeDecorations->setEnabled(mode == WindowUnderCursor);
+    mainWidget->lblIncludeDecorations->setEnabled(mode == WindowUnderCursor);
 }
 
 void KSnapshot::setPreview( const QPixmap &pm )
 {
-    QPixmap pmScaled = pm.scaled( previewWidth(), previewHeight(), Qt::KeepAspectRatio, Qt::SmoothTransformation );
-
     mainWidget->lblImage->setToolTip(
         i18n( "Preview of the snapshot image (%1 x %2)" ,
           pm.width(), pm.height() ) );
 
-    mainWidget->lblImage->setPreview( pmScaled );
+    mainWidget->lblImage->setPreview(pm);
     mainWidget->lblImage->adjustSize();
 }
 
@@ -599,6 +829,16 @@ void KSnapshot::setIncludeDecorations( bool b )
     mainWidget->cbIncludeDecorations->setChecked(b);
 }
 
+void KSnapshot::setIncludePointer(bool enabled)
+{
+    mainWidget->cbIncludePointer->setChecked(enabled);
+}
+
+bool KSnapshot::includePointer() const
+{
+    return mainWidget->cbIncludePointer->isChecked();
+}
+
 void KSnapshot::setMode( int mode )
 {
     mainWidget->comboMode->setCurrentIndex(mode);
