@@ -1,21 +1,24 @@
-/* ============================================================
+/** ===========================================================
+ * @file
  *
- * This file is a part of kipi-plugins project
- * http://www.kipi-plugins.org
+ * This file is a part of digiKam project
+ * <a href="http://www.digikam.org">http://www.digikam.org</a>
  *
- * Date        : 2008-10-09
- * Description : internal private container for KDcraw
+ * @date   2008-10-09
+ * @brief  internal private container for KDcraw
  *
- * Copyright (C) 2008-2010 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * @author Copyright (C) 2008-2010 by Gilles Caulier
+ *         <a href="mailto:caulier dot gilles at gmail dot com">caulier dot gilles at gmail dot com</a>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
  * Public License as published by the Free Software Foundation;
- * either version 2, or (at your option) any later version.
+ * either version 2, or (at your option)
+ * any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * ============================================================ */
@@ -46,22 +49,22 @@ class KDcraw;
 
 extern "C"
 {
-    int callbackForLibRaw(void *data, enum LibRaw_progress p, int iteration, int expected);
+    int callbackForLibRaw(void* data, enum LibRaw_progress p, int iteration, int expected);
 }
 
-class KDcrawPriv
+class KDcraw::KDcrawPriv
 {
 
 public:
 
-    KDcrawPriv(KDcraw *p);
+    KDcrawPriv(KDcraw* p);
     ~KDcrawPriv();
 
 public:
 
-    static void createPPMHeader(QByteArray& imgData, libraw_processed_image_t *img);
+    static void createPPMHeader(QByteArray& imgData, libraw_processed_image_t* img);
 
-    static void fillIndentifyInfo(LibRaw *raw, DcrawInfoContainer& identify);
+    static void fillIndentifyInfo(LibRaw* raw, DcrawInfoContainer& identify);
 
     int progressCallback(enum LibRaw_progress p, int iteration, int expected);
 
@@ -72,7 +75,9 @@ private:
 
     double  m_progress;
 
-    KDcraw *m_parent;
+    KDcraw* m_parent;
+
+    friend class KDcraw;
 };
 
 }  // namespace KDcrawIface
