@@ -1,23 +1,26 @@
-/* ============================================================
+/** ===========================================================
+ * @file
  *
- * This file is a part of kipi-plugins project
- * http://www.kipi-plugins.org
+ * This file is a part of digiKam project
+ * <a href="http://www.digikam.org">http://www.digikam.org</a>
  *
- * Date        : 2006-09-15
- * Description : Exiv2 library interface for KDE
- *               Iptc manipulation methods
+ * @date   2006-09-15
+ * @brief  Iptc manipulation methods
  *
- * Copyright (C) 2006-2010 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2006-2010 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
+ * @author Copyright (C) 2006-2010 by Gilles Caulier
+ *         <a href="mailto:caulier dot gilles at gmail dot com">caulier dot gilles at gmail dot com</a>
+ * @author Copyright (C) 2006-2010 by Marcel Wiesweg
+ *         <a href="mailto:marcel dot wiesweg at gmx dot de">marcel dot wiesweg at gmx dot de</a>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
  * Public License as published by the Free Software Foundation;
- * either version 2, or (at your option) any later version.
+ * either version 2, or (at your option)
+ * any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * ============================================================ */
@@ -43,8 +46,8 @@ bool KExiv2::canWriteIptc(const QString& filePath)
     catch( Exiv2::Error& e )
     {
         std::string s(e.what());
-        kDebug(51003) << "Cannot check Iptc access mode using Exiv2 (Error #"
-                      << e.code() << ": " << s.c_str() << ")" << endl;
+        kDebug() << "Cannot check Iptc access mode using Exiv2 (Error #"
+                      << e.code() << ": " << s.c_str() << ")";
     }
 
     return false;
@@ -84,7 +87,7 @@ QByteArray KExiv2::getIptc(bool addIrbHeader) const
 #if (EXIV2_TEST_VERSION(0,10,0))
                 c2 = Exiv2::Photoshop::setIptcIrb(0, 0, iptc);
 #else
-                kDebug(51003) << "Exiv2 version is to old. Cannot add Irb header to Iptc metadata" << endl;
+                kDebug() << "Exiv2 version is to old. Cannot add Irb header to Iptc metadata";
                 return QByteArray();
 #endif
             }
@@ -105,7 +108,7 @@ QByteArray KExiv2::getIptc(bool addIrbHeader) const
     catch( Exiv2::Error& e )
     {
         if (!d->filePath.isEmpty())
-            kDebug(51003) << "From file " << d->filePath.toAscii().constData() << endl;
+            kDebug() << "From file " << d->filePath.toAscii().constData();
 
         d->printExiv2ExceptionError("Cannot get Iptc data using Exiv2 ",e);
     }
@@ -133,7 +136,7 @@ bool KExiv2::setIptc(const QByteArray& data) const
     catch( Exiv2::Error& e )
     {
         if (!d->filePath.isEmpty())
-            kDebug(51003) << "From file " << d->filePath.toAscii().constData() << endl;
+            kDebug() << "From file " << d->filePath.toAscii().constData();
 
         d->printExiv2ExceptionError("Cannot set Iptc data using Exiv2 ", e);
     }
@@ -420,8 +423,8 @@ bool KExiv2::setIptcTagsStringList(const char* iptcTagName, int maxSize,
         QStringList oldvals = oldValues;
         QStringList newvals = newValues;
 
-        kDebug(51003) << d->filePath.toAscii().constData() << " : " << iptcTagName
-                      << " => " << newvals.join(",").toAscii().constData() << endl;
+        kDebug() << d->filePath.toAscii().constData() << " : " << iptcTagName
+                      << " => " << newvals.join(",").toAscii().constData();
 
         // Remove all old values.
         Exiv2::IptcData iptcData(d->iptcMetadata());
@@ -510,8 +513,8 @@ bool KExiv2::setIptcKeywords(const QStringList& oldKeywords, const QStringList&
         QStringList oldkeys = oldKeywords;
         QStringList newkeys = newKeywords;
 
-        kDebug(51003) << d->filePath.toAscii().constData()
-                      << " ==> Iptc Keywords: " << newkeys.join(",").toAscii().constData() << endl;
+        kDebug() << d->filePath.toAscii().constData()
+                      << " ==> Iptc Keywords: " << newkeys.join(",").toAscii().constData();
 
         // Remove all old keywords.
         Exiv2::IptcData iptcData(d->iptcMetadata());
