@@ -31,6 +31,7 @@
 #include <qtimer.h>
 #include <QtGui/QPrinter>
 #include <QtGui/QPrintDialog>
+#include <QScrollBar>
 
 #include <kvbox.h>
 #include <kaboutapplicationdialog.h>
@@ -55,6 +56,7 @@
 #include <kio/job.h>
 #include <kicon.h>
 #include <kfilterdev.h>
+#include <kfilterbase.h>
 #if 0
 #include <knewstuff2/engine.h>
 #endif
@@ -88,6 +90,7 @@
 #include "core/fileprinter.h"
 
 #include <cstdio>
+#include <memory>
 
 class FileKeeper
 {
@@ -163,20 +166,29 @@ static QString compressedMimeFor( const QString& mime_to_check )
     static QHash< QString, QString > compressedMimeMap;
     if ( compressedMimeMap.isEmpty() )
     {
+        std::auto_ptr< KFilterBase > f;
         compressedMimeMap[ QString::fromLatin1( "application/x-gzip" ) ] =
             QString::fromLatin1( "application/x-gzip" );
-        compressedMimeMap[ QString::fromLatin1( "application/x-bzip" ) ] =
-            QString::fromLatin1( "application/x-bzip" );
-        compressedMimeMap[ QString::fromLatin1( "application/x-bzpdf" ) ] =
-            QString::fromLatin1( "application/x-bzip" );
-        compressedMimeMap[ QString::fromLatin1( "application/x-bzpostscript" ) ] =
-            QString::fromLatin1( "application/x-bzip" );
-        compressedMimeMap[ QString::fromLatin1( "application/x-bzdvi" ) ] =
-            QString::fromLatin1( "application/x-bzip" );
         compressedMimeMap[ QString::fromLatin1( "image/x-gzeps" ) ] =
             QString::fromLatin1( "application/x-gzip" );
-        compressedMimeMap[ QString::fromLatin1( "image/x-bzeps" ) ] =
-            QString::fromLatin1( "application/x-bzip" );
+        // check we can read bzip2-compressed files
+        f.reset( KFilterBase::findFilterByMimeType( QString::fromLatin1( "application/x-bzip" ) ) );
+        if ( f.get() )
+        {
+            const QString app_bzip( QString::fromLatin1( "application/x-bzip" ) );
+            compressedMimeMap[ app_bzip ] = app_bzip;
+            compressedMimeMap[ QString::fromLatin1( "application/x-bzpdf" ) ] = app_bzip;
+            compressedMimeMap[ QString::fromLatin1( "application/x-bzpostscript" ) ] = app_bzip;
+            compressedMimeMap[ QString::fromLatin1( "application/x-bzdvi" ) ] = app_bzip;
+            compressedMimeMap[ QString::fromLatin1( "image/x-bzeps" ) ] = app_bzip;
+        }
+        // check we can read XZ-compressed files
+        f.reset( KFilterBase::findFilterByMimeType( QString::fromLatin1( "application/x-xz" ) ) );
+        if ( f.get() )
+        {
+            const QString app_xz( QString::fromLatin1( "application/x-xz" ) );
+            compressedMimeMap[ app_xz ] = app_xz;
+        }
     }
     QHash< QString, QString >::const_iterator it = compressedMimeMap.constFind( mime_to_check );
     if ( it != compressedMimeMap.constEnd() )
@@ -241,8 +253,9 @@ m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args
 
     QDBusConnection::sessionBus().registerObject("/okular", this, QDBusConnection::ExportScriptableSlots);
 
-    // connect the started signal to tell the job the mimetypes we like
-    connect(this, SIGNAL(started(KIO::Job *)), this, SLOT(setMimeTypes(KIO::Job *)));
+    // connect the started signal to tell the job the mimetypes we like,
+    // and get some more information from it
+    connect(this, SIGNAL(started(KIO::Job *)), this, SLOT(slotJobStarted(KIO::Job *)));
 
     // connect the completed signal so we can put the window caption when loading remote files
     connect(this, SIGNAL(completed()), this, SLOT(setWindowTitleFromDocument()));
@@ -356,18 +369,18 @@ m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args
     rightLayout->addWidget( m_pageView );
     m_findBar = new FindBar( m_document, rightContainer );
     rightLayout->addWidget( m_findBar );
-    QWidget * bottomBar = new QWidget( rightContainer );
-    QHBoxLayout * bottomBarLayout = new QHBoxLayout( bottomBar );
-    m_pageSizeLabel = new PageSizeLabel( bottomBar, m_document );
+    m_bottomBar = new QWidget( rightContainer );
+    QHBoxLayout * bottomBarLayout = new QHBoxLayout( m_bottomBar );
+    m_pageSizeLabel = new PageSizeLabel( m_bottomBar, m_document );
     bottomBarLayout->setMargin( 0 );
     bottomBarLayout->setSpacing( 0 );
     bottomBarLayout->addWidget( m_pageSizeLabel->antiWidget() );
     bottomBarLayout->addItem( new QSpacerItem( 5, 5, QSizePolicy::Expanding, QSizePolicy::Minimum ) );
-    m_miniBar = new MiniBar( bottomBar, m_document );
+    m_miniBar = new MiniBar( m_bottomBar, m_document );
     bottomBarLayout->addWidget( m_miniBar );
     bottomBarLayout->addItem( new QSpacerItem( 5, 5, QSizePolicy::Expanding, QSizePolicy::Minimum ) );
     bottomBarLayout->addWidget( m_pageSizeLabel );
-    rightLayout->addWidget( bottomBar );
+    rightLayout->addWidget( m_bottomBar );
 
     connect( m_reviewsWidget, SIGNAL( setAnnotationWindow( Okular::Annotation* ) ),
         m_pageView, SLOT( setAnnotationWindow( Okular::Annotation* ) ) );
@@ -422,13 +435,15 @@ m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args
     connect( m_progressWidget, SIGNAL( nextPage() ), m_nextPage, SLOT( trigger() ) );
 #endif
 
-    m_firstPage = KStandardAction::firstPage( this, SLOT( slotGotoFirst() ), ac );
-    ac->addAction("first_page", m_firstPage);
-    m_firstPage->setWhatsThis( i18n( "Moves to the first page of the document" ) );
+    m_beginningOfDocument = KStandardAction::firstPage( this, SLOT( slotGotoFirst() ), ac );
+    ac->addAction("first_page", m_beginningOfDocument);
+    m_beginningOfDocument->setText(i18n( "Beginning of the document"));
+    m_beginningOfDocument->setWhatsThis( i18n( "Moves to the beginning of the document" ) );
 
-    m_lastPage = KStandardAction::lastPage( this, SLOT( slotGotoLast() ), ac );
-    ac->addAction("last_page",m_lastPage);
-    m_lastPage->setWhatsThis( i18n( "Moves to the last page of the document" ) );
+    m_endOfDocument = KStandardAction::lastPage( this, SLOT( slotGotoLast() ), ac );
+    ac->addAction("last_page",m_endOfDocument);
+    m_endOfDocument->setText(i18n( "End of the document"));
+    m_endOfDocument->setWhatsThis( i18n( "Moves to the end of the document" ) );
 
     // we do not want back and next in history in the dummy mode
     m_historyBack = 0;
@@ -504,6 +519,12 @@ m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args
     m_showLeftPanel->setChecked( Okular::Settings::showLeftPanel() );
     slotShowLeftPanel();
 
+    m_showBottomBar = ac->add<KToggleAction>("show_bottombar");
+    m_showBottomBar->setText(i18n( "Show &Page Bar"));
+    connect( m_showBottomBar, SIGNAL( toggled( bool ) ), this, SLOT( slotShowBottomBar() ) );
+    m_showBottomBar->setChecked( Okular::Settings::showBottomBar() );
+    slotShowBottomBar();
+
     QAction * importPS = ac->addAction("import_ps");
     importPS->setText(i18n("&Import PostScript as PDF..."));
     importPS->setIcon(KIcon("document-import"));
@@ -633,6 +654,7 @@ Part::~Part()
     delete m_pageView;
     delete m_thumbnailList;
     delete m_miniBar;
+    delete m_bottomBar;
 #ifdef OKULAR_ENABLE_MINIBAR
     delete m_progressWidget;
 #endif
@@ -714,12 +736,22 @@ void Part::openUrlFromBookmarks(const KUrl &_url)
         openUrl( url );
 }
 
-void Part::setMimeTypes(KIO::Job *job)
+void Part::slotJobStarted(KIO::Job *job)
 {
     if (job)
     {
         QStringList supportedMimeTypes = m_document->supportedMimeTypes();
         job->addMetaData("accept", supportedMimeTypes.join(", ") + ", */*;q=0.5");
+
+        connect(job, SIGNAL(result(KJob*)), this, SLOT(slotJobFinished(KJob*)));
+    }
+}
+
+void Part::slotJobFinished(KJob *job)
+{
+    if ( job->error() == KIO::ERR_USER_CANCELED )
+    {
+        m_pageView->noticeMessage( i18n( "The loading of %1 has been canceled.", realUrl().pathOrUrl() ) );
     }
 }
 
@@ -736,10 +768,6 @@ void Part::loadCancelled(const QString &reason)
         {
             KMessageBox::error( widget(), i18n("Could not open %1. Reason: %2", url().prettyUrl(), reason ) );
         }
-        else
-        {
-            KMessageBox::error( widget(), i18n("Could not open %1", url().prettyUrl() ) );
-        }
     }
 }
 
@@ -792,14 +820,7 @@ void Part::notifySetup( const QVector< Okular::Page * > & /*pages*/, int setupFl
 
 void Part::notifyViewportChanged( bool /*smoothMove*/ )
 {
-    // update actions if the page is changed
-    static int lastPage = -1;
-    int viewportPage = m_document->viewport().pageNumber;
-    if ( viewportPage != lastPage )
-    {
-        updateViewActions();
-        lastPage = viewportPage;
-    }
+    updateViewActions();
 }
 
 void Part::notifyPageChanged( int page, int flags )
@@ -1060,6 +1081,10 @@ bool Part::openUrl(const KUrl &_url)
 
         setWindowTitleFromDocument();
     }
+    else
+    {
+        KMessageBox::error( widget(), i18n("Could not open %1", url.pathOrUrl() ) );
+    }
 
     return openOk;
 }
@@ -1150,6 +1175,14 @@ void Part::slotShowLeftPanel()
     m_sidebar->setSidebarVisibility( showLeft );
 }
 
+void Part::slotShowBottomBar()
+{
+    const bool showBottom = m_showBottomBar->isChecked();
+    Okular::Settings::setShowBottomBar( showBottom );
+    Okular::Settings::self()->writeConfig();
+    // show/hide bottom bar
+    m_bottomBar->setVisible( showBottom );
+}
 
 void Part::slotFileDirty( const QString& path )
 {
@@ -1160,10 +1193,10 @@ void Part::slotFileDirty( const QString& path )
     // written to the file.
     if ( path == localFilePath() )
     {
-        // Only start watching the file in case if it wasn't removed, thus was modified
+        // Only start watching the file in case if it wasn't removed
         if (QFile::exists(localFilePath()))
             m_dirtyHandler->start( 750 );
-	else
+        else
             m_fileWasRemoved = true;
     }
     else
@@ -1240,13 +1273,52 @@ void Part::updateViewActions()
     bool opened = m_document->pages() > 0;
     if ( opened )
     {
-        bool atBegin = m_document->currentPage() < 1;
-        bool atEnd = m_document->currentPage() >= (m_document->pages() - 1);
         m_gotoPage->setEnabled( m_document->pages() > 1 );
-        m_firstPage->setEnabled( !atBegin );
-        m_prevPage->setEnabled( !atBegin );
-        m_lastPage->setEnabled( !atEnd );
-        m_nextPage->setEnabled( !atEnd );
+        
+        // Check if you are at the beginning or not
+        if (m_document->currentPage() != 0)
+        {
+            m_beginningOfDocument->setEnabled( true );
+            m_prevPage->setEnabled( true );
+        }
+        else 
+        {
+            if (m_pageView->verticalScrollBar()->value() != 0)
+            {
+                // The page isn't at the very beginning
+                m_beginningOfDocument->setEnabled( true );
+            }
+            else
+            {
+                // The page is at the very beginning of the document
+                m_beginningOfDocument->setEnabled( false );
+            }
+            // The document is at the first page, you can go to a page before
+            m_prevPage->setEnabled( false );
+        }
+        
+        if (m_document->pages() == m_document->currentPage() + 1 )
+        {
+            // If you are at the end, disable go to next page
+            m_nextPage->setEnabled( false );
+            if (m_pageView->verticalScrollBar()->value() == m_pageView->verticalScrollBar()->maximum())
+            {
+                // If you are the end of the page of the last document, you can't go to the last page
+                m_endOfDocument->setEnabled( false );
+            }
+            else 
+            {
+                // Otherwise you can move to the endif
+                m_endOfDocument->setEnabled( true );
+            }
+        }
+        else 
+        {
+            // If you are not at the end, enable go to next page
+            m_nextPage->setEnabled( true );
+            m_endOfDocument->setEnabled( true );
+        }
+
         if (m_historyBack) m_historyBack->setEnabled( !m_document->historyAtBegin() );
         if (m_historyNext) m_historyNext->setEnabled( !m_document->historyAtEnd() );
         m_reload->setEnabled( true );
@@ -1256,8 +1328,8 @@ void Part::updateViewActions()
     else
     {
         m_gotoPage->setEnabled( false );
-        m_firstPage->setEnabled( false );
-        m_lastPage->setEnabled( false );
+        m_beginningOfDocument->setEnabled( false );
+        m_endOfDocument->setEnabled( false );
         m_prevPage->setEnabled( false );
         m_nextPage->setEnabled( false );
         if (m_historyBack) m_historyBack->setEnabled( false );
@@ -1385,15 +1457,25 @@ void Part::slotNextPage()
 
 void Part::slotGotoFirst()
 {
-    if ( m_document->isOpened() )
+    if ( m_document->isOpened() ) {
         m_document->setViewportPage( 0 );
+        m_beginningOfDocument->setEnabled( false );
+    }
 }
 
 
 void Part::slotGotoLast()
 {
     if ( m_document->isOpened() )
-        m_document->setViewportPage( m_document->pages() - 1 );
+    {
+        DocumentViewport endPage(m_document->pages() -1 );
+        endPage.rePos.enabled = true;
+        endPage.rePos.normalizedX = 0;
+        endPage.rePos.normalizedY = 1;
+        endPage.rePos.pos = Okular::DocumentViewport::TopLeft;
+        m_document->setViewport(endPage);
+        m_endOfDocument->setEnabled(false);
+    }
 }
 
 
@@ -2138,12 +2220,12 @@ bool Part::handleCompressed( QString &destpath, const QString &path, const QStri
         return false;
     }
 
-    QByteArray buf(1024, '\0');
+    char buf[65536];
     int read = 0, wrtn = 0;
 
-    while ((read = filterDev->read(buf.data(), buf.size())) > 0)
+    while ((read = filterDev->read(buf, sizeof(buf))) > 0)
     {
-        wrtn = newtempfile->write(buf.data(), read);
+        wrtn = newtempfile->write(buf, read);
         if ( read != wrtn )
             break;
     }
