@@ -135,7 +135,7 @@ PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc
     Q_UNUSED( parent )
     setAttribute( Qt::WA_DeleteOnClose );
     setAttribute( Qt::WA_OpaquePaintEvent );
-    setObjectName( "presentationWidget" );
+    setObjectName( QLatin1String( "presentationWidget" ) );
     QString caption = doc->metaData( "DocumentTitle" ).toString();
     if ( caption.trimmed().isEmpty() )
         caption = doc->currentDocument().fileName();
@@ -147,7 +147,7 @@ PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc
 
     // create top toolbar
     m_topBar = new PresentationToolBar( this );
-    m_topBar->setObjectName( "presentationBar" );
+    m_topBar->setObjectName( QLatin1String( "presentationBar" ) );
     m_topBar->setIconSize( QSize( 32, 32 ) );
     m_topBar->setMovable( false );
     m_topBar->layout()->setMargin(0);
@@ -1057,19 +1057,12 @@ QRect PresentationWidget::routeMouseDrawingEvent( QMouseEvent * e )
     const QRect & geom = m_frames[ m_frameIndex ]->geometry;
     const Okular::Page * page = m_frames[ m_frameIndex ]->page;
 
-    AnnotatorEngine::EventType eventType = AnnotatorEngine::Press;
-    if ( e->type() == QEvent::MouseMove )
-        eventType = AnnotatorEngine::Move;
-    else if ( e->type() == QEvent::MouseButtonRelease )
-        eventType = AnnotatorEngine::Release;
+    AnnotatorEngine::EventType eventType;
+    AnnotatorEngine::Button button;
+
+    // figure out the event type and button
+    AnnotatorEngine::decodeEvent( e, &eventType, &button );
 
-    // find out the pressed button
-    AnnotatorEngine::Button button = AnnotatorEngine::None;
-    Qt::MouseButtons buttonState = ( eventType == AnnotatorEngine::Move ) ? e->buttons() : e->button();
-    if ( buttonState == Qt::LeftButton )
-        button = AnnotatorEngine::Left;
-    else if ( buttonState == Qt::RightButton )
-        button = AnnotatorEngine::Right;
     static bool hasclicked = false;
     if ( eventType == AnnotatorEngine::Press )
         hasclicked = true;
